"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidatorAbstract = exports.registerPostRules = exports.registerRules = void 0;
const message_parser_util_js_1 = require("./utils/message-parser.util.js");
const ops_util_js_1 = require("./utils/ops.util.js");
const obj_util_js_1 = require("./utils/obj.util.js");
const rules_parser_util_js_1 = require("./utils/rules-parser.util.js");
const MessagesProvider = __importStar(require("./messages/provider.js"));
const config = __importStar(require("./config.js"));
let RulesProvider = {};
let PostRulesProvider = {};
/**
 * registerRules will replace old registered rules
 * @param rules validation rules
 */
function registerRules(rules) {
    // Object.keys(rules).forEach((rule) => {
    //   RulesProvider[rule] = rules[rule];
    // });
    // return RulesProvider;
    return RulesProvider = rules;
}
exports.registerRules = registerRules;
/**
 * registerPostRules will replace old post validation rules
 * @param rules post validation rules
 */
function registerPostRules(rules) {
    return PostRulesProvider = rules;
}
exports.registerPostRules = registerPostRules;
// you can change this value with static method
let shouldBreakIfFailed = true;
class ValidatorAbstract {
    /**
     * init validator
     * @param inputs
     * @param rules
     * @param customMessages
     */
    constructor(inputs = {}, rules = {}, customMessages = {}) {
        this.inputs = inputs;
        this.rules = rules;
        this.customMessages = customMessages;
        this.implicitRules = config.get('implicitRules');
        // validation errors collection
        this.errors = {};
        // local custom attributes collection
        this.localNiceNames = {};
        // do we have custom messages?
        this.hasCustomMessages = false;
        // notationMap: any = {};
        // notationVals: any = {};
        // do we have nested rules?
        this.hasNestedRules = false;
        // rules collection
        this.parsedRulesCollection = {};
        this.lang = config.get('lang');
        // post validation rules collection
        this.postRules = [];
        // break rules validation loop when failed
        this.breakWhenFailed = shouldBreakIfFailed;
        // release attribute from futher validation rules
        this.shouldRelease = false;
        this.implicitInputs = {};
        this.hasCustomMessages = Object.keys(customMessages).length > 0;
        this.parse();
    }
    /**
     * globally should break/bail on failed validation or not
     * @param {boolean} sure
     */
    static bailable(sure) {
        shouldBreakIfFailed = sure;
    }
    /**
     * enable/disable multiple errors on current instance only
     * @param {boolean} sure
     */
    bail(sure) {
        this.breakWhenFailed = sure;
    }
    /**
     * release attribute from rules validation
     */
    release() {
        this.shouldRelease = true;
    }
    /**
     * check is instance is bailable or not
     * @returns {boolean}
     */
    isBailable() {
        return this.breakWhenFailed;
    }
    /**
     * allows a custom rule to be added as an implicit rule for the instance only
     * @param {String} ruleName
     */
    addImplicitRule(ruleName) {
        this.implicitRules.push(ruleName);
    }
    addRules(rules = {}) {
        this.rules = rules;
        this.parseRules();
    }
    async postRuleApply(rule) {
        if (rule.rule === 'function') {
            // eslint-disable-next-line no-return-await
            return await rule.handler(this);
        }
        // eslint-disable-next-line no-return-await
        return await PostRulesProvider[rule.rule](rule, this);
    }
    /**
    * add post rule
    *
    * post rule is applied to whole input and is used to check constraints
    * across multiple fields
    *
    * @param {*} rule
   */
    addPostRule(rule) {
        if (typeof rule === 'function') {
            this.postRules.push({
                rule: 'function',
                handler: rule,
            });
            return;
        }
        const ruleArray = rule.split(':', 2);
        const ruleName = ruleArray[0];
        const ruleFields = ruleArray[1].split(','); // there always be a list of fields
        this.postRules.push({
            rule: ruleName,
            params: ruleFields,
        });
    }
    /**
      * add set of post rules
      *
      * @param {string[]} postRulesObj
      */
    addPostRules(postRulesObj) {
        if (!Array.isArray(postRulesObj)) {
            postRulesObj = postRulesObj.split('|');
        }
        postRulesObj.map((rule) => this.addPostRule(rule));
    }
    /**
     * parse provided rules and inputs
     */
    parse() {
        this.parseRules();
        this.parseInputs();
    }
    parseRules() {
        let attr;
        for (attr of Object.keys(this.rules)) {
            if (attr === '*') {
                this.addPostRules(this.rules[attr]);
                return;
            }
            if (attr.indexOf(".")) {
                this.hasNestedRules = true;
            }
            const attrRules = this.rules[attr];
            if (typeof attrRules === 'string') {
                this.parsedRulesCollection[attr] = rules_parser_util_js_1.parseStringNotationRules(RulesProvider, attrRules);
            }
            else {
                attrRules.forEach((strRule, index) => {
                    if (typeof strRule === 'string') {
                        attrRules[index] = rules_parser_util_js_1.parseStringRule(RulesProvider, strRule);
                    }
                });
                // @ts-ignore
                this.parsedRulesCollection[attr] = attrRules;
            }
            // sort rules as
            this.parsedRulesCollection[attr].sort((obj) => {
                return (this.implicitRules.indexOf(obj.name) >= 0) ? -1 : 1;
            });
        }
    }
    fillMissingAttributes(key) {
        const data = {};
        const [frontKey] = key.split('.');
        data[frontKey] = this.inputs[frontKey];
        ops_util_js_1.fillMissingSpots(data, key);
        // this.implicitInputs[frontKey] = data[frontKey];
        const { notationMap } = obj_util_js_1.getValuesByWildCardStringNotation(data);
        const keys = Object.keys(notationMap);
        const len = keys.length;
        let i = 0;
        for (i; i < len; i += 1) {
            const key = keys[i];
            const attrRules = this.parsedRulesCollection[key];
            if (attrRules /*&& key.indexOf('*') >= 0*/) {
                notationMap[key].forEach((attrName) => {
                    this.parsedRulesCollection[attrName] = attrRules;
                });
            }
        }
    }
    parseInputs() {
        if (!this.hasNestedRules) {
            return;
        }
        // const flat = (data: any, prepend = '') => {
        //   const results: any = {};
        //   Object.keys(data).forEach(key => {
        //     const value = data[key];
        //     if (typeof value === 'object') {
        //       results.push(...results, ...flat(value, prepend + key + '.'));
        //     } else {
        //       results[prepend + key] = value;
        //     }
        //   });
        //   return results;
        // }
        Object.keys(this.rules).forEach((key) => {
            if (key.indexOf('*') >= 0) {
                this.fillMissingAttributes(key);
            }
        });
        // const { notationMap, notationsVals } = getValuesByWildCardStringNotation(
        //   this.inputs, this.rules,
        // );
        // this.notationMap = notationMap;
        // this.notationVals = notationsVals;
        // const keys = Object.keys(this.notationMap);
        // const len = keys.length;
        // let i = 0;
        // for (i; i < len; i += 1) {
        //   const key = keys[i];
        //   const attrRules: Array<ValidationRuleContract> = this.parsedRulesCollection[key];
        //   if (attrRules && key.indexOf('*') >= 0) {
        //     this.notationMap[key].forEach((attrName: string) => {
        //       this.parsedRulesCollection[attrName] = attrRules;
        //     });
        //   }
        // }
    }
    /**
     * apply this set of filters to inputs
     * @param filters set of filters
     */
    applyFilters(filters) {
        // future ref
    }
    /**
    * apply post validation rules
    * @param rules post rules
    */
    applyPostRules(rules) {
        // to-do
    }
    /**
     * validate inputs againest rules
     */
    async validate(inputs) {
        if (inputs) {
            this.inputs = inputs;
            this.parseInputs();
        }
        const keys = Object.keys(this.parsedRulesCollection);
        const len = keys.length;
        let i = 0;
        const promises = [];
        for (i; i < len; i += 1) {
            const attrName = keys[i];
            if (attrName.indexOf('*') < 0) {
                const attrRules = this.parsedRulesCollection[attrName];
                if (attrRules) {
                    promises.push(this.validateAttribute(attrName, attrRules));
                }
            }
        }
        // post validation rules
        this.postRules.forEach((postRule) => {
            promises.push(this.postRuleApply(postRule));
        });
        await Promise.all(promises);
        return !this.hasErrors();
    }
    /**
    * apply rules on attribute
    * @param attrName attribute name
    * @param attrRules attribute rules
    */
    async validateAttribute(attrName, attrRules) {
        let i = 0;
        let len = attrRules.length;
        for (i; i < len; i += 1) {
            const validationRule = attrRules[i];
            let attrValue = this.attributeValue(attrName);
            if (
            // no implicit rule and attribute value is empty
            (this.implicitRules.indexOf(validationRule.name) < 0 &&
                ops_util_js_1.reallyEmpty(attrValue))
            // attribute can be nullable
            // (validationRule.name === "nullable" && attrValue === null) ||
            // // attribute will only be validated if presents
            // (validationRule.name === "sometimes" &&
            //   this.isAttributePresent(attrName) === false)
            ) {
                if (this.isBailable()) {
                    // console.log(' here i m ...');
                    return;
                }
                attrValue = '';
            }
            // console.log(this.breakWhenFailed, this.isBailable());
            const passed = await validationRule.handler(attrValue, this, attrName);
            this.shouldRelease = false;
            // console.log('attr', attrName, validationRule.name, passed);
            if (!passed) {
                this.createAttributeError({
                    attrName,
                    attrValue,
                    ruleName: validationRule.name,
                    ruleArgs: validationRule.args,
                });
                if (this.isBailable()) {
                    return;
                }
            }
        }
    }
    /**
    * this will create error object for attribute
    * @param params info object
    */
    createAttributeError(params) {
        const { attrName, ruleName } = params;
        const error = {
            rule: ruleName,
            message: this.createAttributeErrorMessage(params),
        };
        if (!this.breakWhenFailed) {
            if (!this.errors[attrName]) {
                this.errors[attrName] = [];
            }
            // @ts-ignore
            this.errors[attrName].push(error);
            return;
        }
        this.errors[attrName] = error;
    }
    /**
     * this will return parsed error message as per rule or input
     * @param params object with attr and rule name, value, args
     */
    createAttributeErrorMessage(params, useDefaultMessage = true) {
        const { attrName, ruleName, attrValue, ruleArgs } = params;
        const messagesCollection = MessagesProvider.messagesRefByLang(this.lang);
        const defaultMessage = messagesCollection.$default;
        let message;
        // check for local scope messages
        if (this.hasCustomMessages) {
            message = this.customMessages[`${attrName}.${ruleName}`] ||
                this.customMessages[ruleName] ||
                this.customMessages[attrName];
        }
        // not found in local scope, check for global scope
        if (!message) {
            message = (messagesCollection.$custom &&
                messagesCollection.$custom[`${attrName}.${ruleName}`]) ||
                (messagesCollection.$custom && messagesCollection.$custom[attrName]) ||
                messagesCollection[ruleName];
            if (useDefaultMessage && !message) {
                message = defaultMessage;
            }
        }
        let attributeName = attrName;
        let niceName;
        // check if we have nice name in local scope
        if (this.localNiceNames[attrName]) {
            niceName = this.localNiceNames[attrName];
        }
        else if (messagesCollection.$niceNames && messagesCollection.$niceNames[attrName]) {
            // check if we have nice name in global scope
            niceName = messagesCollection.$niceNames[attrName];
        }
        return message_parser_util_js_1.messageParser({
            message,
            attrName: attributeName,
            niceName,
            ruleName,
            attrValue,
            ruleArgs,
        });
    }
    /**
    * get attribute value by its name
    * @param attr attribute name
    */
    attributeValue(attr) {
        return obj_util_js_1.getValueByStringNotation(this.inputs, attr);
        // return this.inputs[attr] || this.notationVals[attr];
    }
    /**
     * check if attribute has value
     * @param attr attribute name
     */
    doAttributeHasValue(attr) {
        return this.attributeValue(attr) === undefined ? true : false;
    }
    /**
     * does attribute present in given inputs
     * @param attr attribute name
     */
    isAttributePresent(attr) {
        if (this.inputs[attr]) {
            return true;
        }
        return false;
    }
    /**
     * does we have any dirty/failed input
     */
    hasErrors() {
        return Object.keys(this.errors).length > 0;
    }
    /**
     * get validation errors
     */
    getErrors() {
        return this.errors;
    }
    niceNames(niceNames) {
        this.localNiceNames = niceNames;
    }
}
exports.ValidatorAbstract = ValidatorAbstract;
//# sourceMappingURL=validator-abs.backup.js.map