export * from './contracts.js';
export * as Messages from './messages/index.js';
import * as Rules from './rules/index.js';
import { Validator } from './validator.js';
export { Validator, Rules };
export { MomentAdapter, DateFnsAdapter } from './date/index.js';
import * as config from "./config.js";
export function configure(customConf) {
    config.set(customConf);
}
export function useDateAdapter(dateAdapter) {
    config.modify('dateAdapter', dateAdapter);
}
export function lang(lang) {
    config.modify('lang', lang);
}
export function addImplicitRule(ruleName) {
    config.addImplicitRule(ruleName);
}
export function extend(ruleName, ruleFunc) {
    // @ts-ignore
    Rules[ruleName] = ruleFunc;
}
/* istanbul ignore next */
export function koa() {
    return async (ctx, next) => {
        // @ts-ignore
        ctx.validationErrors = function validationErrors(errors) {
            return {
                body: {
                    message: 'The given data is invalid.',
                    errors,
                },
            };
        };
        ctx.validate = async function validate(rulesArray, inputs, useMessages) {
            const v = new Validator(inputs || Object.assign(Object.assign({}, this.request.body), this.request.files), rulesArray || {}, useMessages || {});
            if (!(await v.validate())) {
                this.throw(422, this.validationErrors(v.errors));
            }
            return v;
        };
        ctx.validator = (inputs, rulesArray, useMessages) => new Validator(inputs, rulesArray, useMessages);
        try {
            await next();
        }
        catch (err) {
            if (err.status && err.status === 422) {
                ctx.type = 'json';
                ctx.status = 422;
                ctx.body = err.body;
                return;
            }
            throw err;
        }
    };
}
/**
 * enable/disable multiple errors output
 * @param {boolean} sure
 */
export function bailable(sure) {
    Validator.bailable(sure);
}
//# sourceMappingURL=index.js.map